/*
 * deviced
 *
 * Copyright (c) 2016 Samsung Electronics Co., Ltd.
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */


#include <stdio.h>
#include <errno.h>

#include "common.h"
#include "devices.h"
#include "device-notifier.h"
#include "edbus-handler.h"
#include "log.h"

static device_notifier_state_e state = DEVICE_NOTIFIER_STATE_STOP;

static DBusMessage *dbus_start_event_handler(E_DBus_Object *obj, DBusMessage *msg)
{
	if (state != DEVICE_NOTIFIER_STATE_START) {
		_D("START event-handler");
		state = DEVICE_NOTIFIER_STATE_START;
		device_notify(DEVICE_NOTIFIER_EVENT_HANDLER, (void *)&state);
	}
	return make_reply_message(msg, 0);
}

static DBusMessage *dbus_stop_event_handler(E_DBus_Object *obj, DBusMessage *msg)
{
	if (state != DEVICE_NOTIFIER_STATE_STOP) {
		_D("STOP event-handler");
		state = DEVICE_NOTIFIER_STATE_STOP;
		device_notify(DEVICE_NOTIFIER_EVENT_HANDLER, (void *)&state);
	}
	return make_reply_message(msg, 0);
}

static DBusMessage *dbus_udev_event_handler(E_DBus_Object *obj, DBusMessage *msg)
{
	DBusError err;
	char *type_str, *argv;
	int argc;

	dbus_error_init(&err);
	if (!dbus_message_get_args(msg, &err,
				DBUS_TYPE_STRING, &type_str,
				DBUS_TYPE_INT32, &argc,
				DBUS_TYPE_STRING, &argv,
				DBUS_TYPE_INVALID)) {
		_E("there is no message");
		goto out;
	}

	if (strncmp(argv, "start", strlen("start")) == 0)
		return dbus_start_event_handler(obj, msg);
	else if (strncmp(argv, "stop", strlen("stop")) == 0)
		return dbus_stop_event_handler(obj, msg);

out:
	return make_reply_message(msg, -EINVAL);
}

static const struct edbus_method edbus_methods[] = {
	{ "EventStart",  NULL,  "i", dbus_start_event_handler },
	{ "EventStop",   NULL,  "i", dbus_stop_event_handler  },
	{ "udev",       "sis",  "i", dbus_udev_event_handler  }, /* deprecated */
};

static void event_handler_init(void *data)
{
	int ret;

	ret = register_edbus_method(DEVICED_PATH_SYSNOTI,
			edbus_methods, ARRAY_SIZE(edbus_methods));
	if (ret < 0)
		_E("fail to init edbus method(%d)", ret);

	state = DEVICE_NOTIFIER_STATE_START;
}

static const struct device_ops event_handler_device_ops = {
	.name     = "event-handler",
	.init     = event_handler_init,
};

DEVICE_OPS_REGISTER(&event_handler_device_ops)
